"""
EMIL EMAIL CLIENT
Copyright (C) 2002  Andreas Prlic
andorsch@gmx.at

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


# new in version 0.3 :
	use the python email module (http://www.python.org/doc/lib/module-email.html)

"""

import Globals
import OFS
from OFS import SimpleItem
import string
from Globals import DTMLFile
from DateTime import DateTime    # to create send_mail time


import email  # use python email library !
from email import _Message

class Email( _Message,SimpleItem.SimpleItem  ):
    "an email message"

    meta_type = "Email"
    manage_options = (
	{'label': 'View',     'action': 'index_html'},	
	{'label': 'Reply',    'action': 'reply_mail'},
	)

    index_html       = DTMLFile('dtml/email_index_html'      , globals())
    
    def __init__(self,id,title,size,uid) :
	"initialize an email object"

    	self.id      = id
	self.title   = title
	# should display the correct size of the mail in zope, but does not work :-/
	self.size    = size
	
	# uid = unique id from email server for this message
	# it is used to keep distinguish read / unread messages from server	
	self.uid     = uid

	# at some point status should be displayed for every email
	self.possible_status = ['read', 'unread', \
				'replied', 'forwarded' , \
				'sent', 'queued']
	self.status  = 'unread'
	
	# this object is not an instance of the MEssage objected from the email module!
	# which is coming from the python email module
	# donnot know why this is necessary, but id does not work otherwise :-/
	self._headers = []
	
	
    def from_server_list(self,message_lst):
	" build up the message from a server_lst "
    	message_str = string.join(message_lst,'\n')

	# cretes a new object
	email_obj  = email.message_from_string(message_str)
	
	# copy all data  into this object
	for key in email_obj.keys() :
	    del self[key]
	    self[key] = email_obj[key]
	
	payload = email_obj.get_payload()
	self.set_payload(payload)
	    
	self.title = " %s %s" % (self['From'],self['Subject'])

    def __getattr__(self, name):
	'''get an attribute from the object'''
    
	txt = self.get(name)
	if txt != None :	    
	    return txt
	return SimpleItem.__getattr__(self, name)	

    def has_attachements(self) :
	" returns 1 if the email has attachements "
	if self.is_multipart() :
	    return 1
	else :
	    return 0

    def number_attachements(self):
	""" returns the number of attachements, '' if there are no """

	if self.has_attachements() :
	    pay = self.get_payload()
	    l = len(pay) - 1
	    return l

	return ''


    def get_attachements(self):
	" returns a list describing the attachements "
	ret = []
	if not self.is_multipart()  :
	    return ret
	else :
	    counter = -1
	    txtcount = 0
	    for msg in self.get_payload() :
		counter = counter + 1
		params = msg.get_params()
		type = params[0][0]
		if type == 'text/plain' :
		    if txtcount == 0 :
			# first txtcouunt is body of message
			txtcount = txtcount + 1
			continue
		# append the filename the list

		lst = ( ('counter', counter), ('filename', msg.get_filename()),('Content-Type',type),)
		#'counter', counter, 'filename', msg.get_filename(),'Content-Type',type]
		ret.append(lst)
	return ret

    def get_attachement(self, number, REQUEST=None,RESPONSE=None) :
	"returns attachment number #... "
	number = int(number)
	
	msg = self.get_payload(number)
	params = msg.get_params()
	type = params[0][0]
	RESPONSE.setHeader('Content-Type', type)
	return msg.get_payload(decode=1) 

	
    def get_body(self) :
	""" returns the body of the messages """

	if self.is_multipart()  :
	   
	    for msg in self.get_payload() :		
		params = msg.get_params()
		type = params[0][0]
		if type == 'text/plain' :
		    return msg.get_payload(decode=1)
	    return ""

	else :
	    # simple mail, no attachement
	    return self.get_payload()


    def set_values(self,
		   title='',		   
		   status='unread'  ) :
	" set values of an email from outside"
		   
	self.title   = title
	self.set_status(status)

    def set_status(self,status) :
	" change the status of an email" 
	if status in self.possible_status :
	    # only change if status = unread
	    
	    if self.status == 'unread' or self.status == 'queued' :
		self.status = status 
	else :
	    raise "unknown_email_status", "unkown status: %s" % status
	
	return ""

    def get_date(self) :
	""" create correct date string for send mail """
	# get differnece to GMT
	gmt = DateTime().toZone('GMT').strftime('%Y%m%d%H%M')
	loc = DateTime().strftime('%Y%m%d%H%M')
	d = long(loc)- long(gmt) 
	
	
	if d < 0 :
	    diff = "-"
	else :
	    diff = "+"

	d = "0" + str(d)
	# if d > 9 then the leading 0 does not cause problems, \
	# since we only slice from the right:
	diff = diff + d[-4:]
	
	date = str(DateTime().strftime('%a, %d %b %Y %H:%M:%S '))
	return date + diff
	
    def send_mail(self,From,To,Subject,Body) :
	""" send an email """
	# assume that MailHost is called MailHost :
	self.set_status('queued')

	# build upe email using the email module :
	self.add_header('Content-Type','text/plain')
	self.add_header('From',From)
	self.add_header('To',To)	
	self.add_header('Subject',Subject)
	self.add_header('User-Agent','Emil_Email_Client')
	date = self.get_date()
	self.add_header('Date',date)
	self.set_payload(Body)

	messageText = self.as_string()
	self.MailHost.send(messageText,subject=Subject)
	self.set_status('sent')
	self.title = "%s %s" % (Subject,To)
  
    def reply_mail(self) :
	" reply message "
	# format body to contain ">" reply symbol still missing

	# call the write mail dtml document. ( it is created from email folder )
	return self.write_mail_html(To=self.From)


    # html formatting functions
    
    def change_larger_smaller(self,line) :
	" format < and > symbols to avoid confusin with html tag "
	line = string.replace(line, "<", "&lt;")
	line = string.replace(line, ">", "&gt;")
	return line


    def highlight_links(self,txt):
	" change appeareance of email, urls "
	ret = ""
	import re
	
	for line in string.split(txt,"\n") :
	    # change emails
	    pat = re.search("([a-zA-Z0-9\._-]*)@([a-zA-Z0-9\._-]*)",line)
	   
	    if  pat != None:
		user  = pat.group(1)
		server= pat.group(2)
		str = "%s@%s" % (user,server)
		#line = string.replace(line,str,'<a href="mailto:%s">%s</a>'% (str,str))
		line = string.replace(line,str,'<a href="write_mail_html?To=%s">%s</a>'\
				      % (str,str))
	    # change urls
	    pat2 = re.search("(http://[a-zA-Z0-9\._\-\/]*)",line)
	    if pat2 != None :
		url = pat2.group(1)
		line = string.replace(line,url,'<a href="%s">%s</a>' % (url,url))
	    ret = ret + line + "\n"

	return ret


    
def manage_addEmail(self,id, title='',message_lst='', size='0 Kb', uid =None, REQUEST=None) :
    " add a email to Zope "

    self._setObject(id, Email(id,title,size,uid))
    
    if REQUEST is not None: return self.manage_main(self,REQUEST)




